/*
 * Created on Oct 26, 2003
 */
package portaview;

import java.util.Observer;

import portaview.model.PortaViewModel;
import portaview.view.AbstractDisplayerView;

/**
 * The general implementation of a viewer controller.  This is used in the 
 * case where one needs to show a preview or full-screen view of an image.
 * Note that this is an controller, not a view.
 * @author William Lee
 */
public abstract class AbstractViewer implements Observer
{
    protected PortaViewModel model = null;
    protected AbstractDisplayerView view = null;

    /**
     * Constructs an AbstractViewer, given the model that that this view observes on.
     * @param m the PortaViewModel that this view observes on.
     */
    public AbstractViewer(PortaViewModel m)
    {
        this.model = m;
    }

    /**
     * @return the PortaView model
     */
    public PortaViewModel getModel()
    {
        return model;
    }

    /**
     * Changes the internal PortaView model for this view.
     * @param model the PortaView model.
     */
    public void setModel(PortaViewModel model)
    {
        // we want to make sure that we unregister the listener
        // to the model and ask the view to do so.
        if (this.model != null)
        {
            this.model.deleteObserver(this);
        }
        this.model = model;
        this.model.addObserver(this);
        view.setPortaViewModel(model);
    }

    /**
     * Moves to the next slide.
     */
    public void nextSlide()
    {
        model.nextSlide();
    }

    /**
     * Moves to the previous slide.
     *
     */
    public void prevSlide()
    {
        model.prevSlide();
    }

    /**
     * Sets the viewer to play.
     *
     */
    public void play()
    {
        model.play();
    }

    /**
     * Sets the viewer to pause.
     */
    public void pause()
    {
        model.pause();
    }
    
    /**
     * @return the view associated with this controller.
     */
    public AbstractDisplayerView getView()
    {
        return view;
    }

    /**
     * Changes the view associated with this controller.
     * @param view the view to set.
     */
    public void setView(AbstractDisplayerView view)
    {
        this.view = view;
    }

}
