/*
 * Created on Sep 19, 2003
 * Copyright (c) 2003 William Lee.  All rights reserved.
 */
package portaview;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;

/**
 * The basic frame that is the parent of all frames.
 * @author <a href="mailto:wwlee1@uiuc.edu">William Lee</a>
 * @version $Id: DefaultFrame.java,v 1.4 2003/12/10 06:18:57 wlee Exp $
 */
public class DefaultFrame extends JFrame
{

	/**
	 * Constructs the default frame with a name
	 * @param name name of frame.
	 */
	public DefaultFrame(String name)
	{
		super(name);
		setDefaultWindowAdapter();
	}
	
	/**
	 * Constructs the default frame
	 */
	public DefaultFrame()
	{
	   this(null);
	}
	
	/**
	 * Position the window in the center of the display.
	 */
	protected void center()
	{
		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		setLocation(
			d.width / 2 - getWidth() / 2,
			d.height / 2 - getHeight() / 2);
	}

	/**
	 * Generates a new WindowAdapter that closes the application.
	 * @return a window listener that just call System.exit(0);
	 */
	protected WindowAdapter getDefaultWindowAdapter()
	{
		WindowAdapter rtn = (new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				System.exit(0);
			}

			public void windowDeiconified(WindowEvent e)
			{
			}

			public void windowIconified(WindowEvent e)
			{
			}
		});
		return rtn;
	}
	
	/**
	 * Set the default window adapater.
	 */
	protected void setDefaultWindowAdapter()
	{
		this.addWindowListener(getDefaultWindowAdapter());
	}

}
