/*
 * Created on Oct 22, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package portaview;

import java.awt.BorderLayout;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Window;
import java.util.Observable;

import javax.swing.JFrame;

import portaview.model.PortaViewModel;
import portaview.view.DisplayerView;

/**
 * Represents a displayer frame and controller that shows the full-screen 
 * view of the picture.  
 *
 * @author William Lee
 */
public class Displayer extends AbstractViewer
{
    private JFrame frame = null;
    private Window win = null;

    /**
     * Constructs a Displayer that observes on the PortaViewModel.
     * @param model the PortaViewModel associated with this Displayer
     */
    public Displayer(PortaViewModel model)
    {
        super(model);
        // this is experimental implementation of the display panel.
        setView(new DisplayerView(model, this));
        
        // Determine if full-screen mode is supported directly
        GraphicsDevice gs = getGraphicsDevice();
        if (!gs.isFullScreenSupported())
        {
            frame = new JFrame();
            //TODO simulate full screen;
        }
        else
        {
            frame = new JFrame(gs.getDefaultConfiguration());
        }
        // Enter full-screen mode
        win = new Window(frame);
        win.add(view, BorderLayout.CENTER);

    }
    
    /**
     * Returns the frame of this displayer.
     * @return
     */
    public JFrame getFrame()
    {
        return frame;
    }

    /**
     * Shows or hides the displayer.
     * @param v show this frame if true, hide otherwise.
     */
    public void setVisible(boolean v)
    {
        if (v)
        {
            getGraphicsDevice().setFullScreenWindow(win);
            frame.setVisible(true);
            win.validate();
        }
        else
        {
            getGraphicsDevice().setFullScreenWindow(null);
        }
    }


    /**
     * Returns the graphics device.
     * @return the graphic deivce used in the full screen view.
     */
    private GraphicsDevice getGraphicsDevice()
    {
        GraphicsEnvironment ge =
            GraphicsEnvironment.getLocalGraphicsEnvironment();
        return (ge.getDefaultScreenDevice());
    }

    /**
     * Does nothing, for the internals will update themselves.
     * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
     */
    public void update(Observable o, Object arg)
    {
    }

}
