/*
 * Created on Oct 6, 2003
 * Copyright (c) 2003.  All rights reserved.
 */
package portaview;

import portaview.server.PortaViewClient;
import portaview.server.PortaViewServer;

/**
 * Registry that contains the static content that everyone else needs.
 * This is an organized way to store shared services that one do not
 * want to pass around as function arguments.
 * 
 * @author <a href="mailto:wwlee1@uiuc.edu">William Lee</a>
 * @version $Id: Registry.java,v 1.4 2003/12/10 06:18:57 wlee Exp $
 */
public class Registry
{
	private static PortaViewServer server = null;
	private static PortaViewClient client = null;
	private static PortaViewMain mainApplication = null;
	private static boolean master = false;

	/**
	 * Initializes the registry.
	 *
	 */
	public static synchronized void reset()
	{
		master = false;
		server = null;
		client = null;
		mainApplication = null;
	}

	/**
	 * @return the PortaView client that can be used to talk to remote PortaViews
	 */
	public static synchronized PortaViewClient getClient()
	{
		return client;
	}

	/**
	 * Sets the PortaView client.
	 * @param client PortaView client.
	 */
	public static synchronized void setClient(PortaViewClient client)
	{
		Registry.client = client;
	}

	/**
	 * @return the PortaView server.
	 */
	public static synchronized PortaViewServer getServer()
	{
		return server;
	}

	/**
	 * Sets the PortaView server.
	 * @param server PortaView server.
	 */
	public static synchronized void setServer(PortaViewServer server)
	{
		Registry.server = server;
	}

	/**
	 * @return true if this PortaView is a master PortaView.
	 */
	public static synchronized boolean isMaster()
	{
		return master;
	}

	/**
	 * Sets the master state for this PortaView.
	 * @param master true if you want to make this PortaView a master PortaView,
     * false otherwise.
	 */
	public static synchronized void setMaster(boolean master)
	{
		Registry.master = master;
	}

    /**
     * @return the main PortaView application
     */
    public static PortaViewMain getMainApplication()
    {
        return mainApplication;
    }

    /**
     * Sets the main application.
     * @param main the main application.
     */
    public static void setMainApplication(PortaViewMain main)
    {
        mainApplication = main;
    }

}
