/*
 * Created on Nov 3, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package portaview;

import java.util.Observable;
import java.util.Observer;

import portaview.model.PortaViewModel;

/**
 * Controller to update the remote PortaView when user assigns an album to it.  
 * This controller is associated with each remote PortaView registered in the master PortaView.  
 * @author William Lee
 */
public class RemoteUpdateController implements Observer
{
    private PortaViewModel model = null;

    /**
     * Constructs the controller with a given PortaViewModel that it observes.
     * @param pvm model to observe.
     */
    public RemoteUpdateController(PortaViewModel pvm)
    {
        model = pvm;
        pvm.addObserver(this);
    }

    /**
     * Update the remote PortaView when the update happens in the PortaViewModel.  Note
     * that if this is the current PortaView, we do nothing, since the model is already
     * updated and we do not have to do this again.
     * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
     */
    public void update(Observable o, Object arg)
    {
        if (!(o instanceof PortaViewModel))
            return;
        PortaViewModel pvm = (PortaViewModel) o;
        if (pvm.equals(Registry.getMainApplication().getThisPortaViewModel()))
        {
            // If the name equals the current portaview do nothing
            return;
        }
        else
        {
            Registry.getClient().updateRemotePortaView(pvm);
        }

    }

    /**
     * @return
     */
    public PortaViewModel getModel()
    {
        return model;
    }

    /**
     * @param model
     */
    public void setModel(PortaViewModel model)
    {
        this.model = model;
    }

}
