/*
 * Created on Oct 12, 2003
 * Copyright (c) 2003.  All rights reserved.
 */
package portaview.model;

import java.io.File;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

/**
 * Represents the album in PortaView.  An album is a folder that contain
 * images on disk.  This object is not used in the MVC model but rather 
 * serves as a class to retrieve photos on disk.
 * 
 * @author <a href="mailto:wwlee1@uiuc.edu">William Lee</a>
 * @version $Id: Album.java,v 1.5 2003/12/10 06:18:57 wlee Exp $
 */
public class Album extends NameModel implements XmlRpcType
{
    public static final String XT_NAME = "name";
    public static final String XT_ICON = "icon";
    public static final String XT_PHOTOS = "photos";
    public static final int ICON_W = 48;
    public static final int ICON_H = 48;

    File directory = null;
    byte[] icon = null;

    /**
     * Creates an album from a directory.
     * @param dir directory for the album.
     */
    public Album(File dir)
    {
        super(dir.getName());
        this.directory = dir;
        Photo p = getPhoto(0);
        if (p != null)
        {
            icon = p.getJpegIcon(ICON_W, ICON_H);
        }
    }

    /**
     * @return the directory for this album.
     */
    public File getDirectory()
    {
        return directory;
    }

    /**
     * Sets the directory.
     * @param directory album directory
     */
    public void setDirectory(File directory)
    {
        this.directory = directory;
    }
    
    /**
     * Lists out the photos inside this album.
     * @return the list of File that represents the photos in this album.
     */
    public List listPhotos()
    {
        if (directory == null) return new Vector();
        File[] listing = directory.listFiles();
        return getJpegs(listing);
    }

    /**
     * Returns the photo with the show name and the sequence id.
     * @param seq the sequence of photo based on the directory's sorting order.
     * @return the photo.
     */
    public Photo getPhoto(int seq)
    {
        if (directory == null)
            return null;
        List jpegs = listPhotos();
        File f = null;
        try
        {
            return new Photo((File) jpegs.get(seq));
        }
        catch (Exception e)
        {
        }
        return null;

    }
    
    /**
     * Returns a photo given the name.
     * @param name name of photo.
     * @return the Photo.
     */
    public Photo getPhoto(String name)
    {
        if (directory == null)
            return null;
        File jpeg = new File(directory, name + ".jpg");
        File f = null;
        try
        {
            return new Photo(jpeg);
        }
        catch (Exception e)
        {
        }
        return null;
    }
    
    /**
     * Returns the list of jpeg bytes according to the file list.
     * 
     * @param listing an array of files.
     * @return List of file with an jpg or JPG extension.
     */
    private List getJpegs(File[] listing)
    {
        List l = new Vector();
        for (int i = 0; i < listing.length; i++)
        {
            if (listing[i].getName().endsWith(".jpg")
                || listing[i].getName().endsWith(".JPG"))
            {
                l.add(listing[i]);
            }

        }
        return l;
    }

    /**
     * Returns a Hashtable of name = the name of album and 
     * icon = byte[] of the album image.
     * @see portaview.model.XmlRpcType#toXmlRpcType()
     */
    public Object toXmlRpcType()
    {
        Hashtable rtn = new Hashtable();
        rtn.put(XT_NAME, getName());
        rtn.put(XT_ICON, this.icon);
        List flist = listPhotos();
        Vector photos = new Vector();
        for (Iterator i = flist.iterator(); i.hasNext();)
        {
            File f = (File)i.next();
            photos.add(f.getName());
        }
        rtn.put(XT_PHOTOS, photos);
        return rtn;
    }
}
