/*
 * Created on Oct 29, 2003
 */
package portaview.model;

import java.awt.Image;
import java.net.URL;

import org.apache.xmlrpc.AsyncCallback;

import portaview.Log;
import portaview.util.ImageUtils;

/**
 * Abstract implementation of an image model.
 * @author William Lee
 */
public abstract class ImageModel
    extends DomainModel
    implements AsyncCallback
{
    private Image image = null;

    /**
     * Constructs an ImageModel associated with no image.
     */
    public ImageModel()
    {
        this(null);
    }

    /**
     * Constructs an ImageModel associated with a given image.
     * @param img Image object.
     */
    public ImageModel(Image img)
    {
        this.image = img;
    }

    /**
     * @return the image
     */
    public synchronized Image getImage()
    {
        return image;
    }

    /**
     * Sets the image
     * @param image the image to set.
     */
    public synchronized void setImage(Image image)
    {
        this.image = image;
    }

    /**
     * Handles the result for the async call to master when 
     * retrieving the picture.
     * 
     * @see org.apache.xmlrpc.AsyncCallback#handleResult(java.lang.Object, java.net.URL, java.lang.String)
     */
    public synchronized void handleResult(
        Object result,
        URL url,
        String method)
    {
        byte[] img = (byte[]) result;
        this.image = ImageUtils.getImage(img);
        // Now, if we get the image, go ahead and notify the view changes.
        setChanged();
        notifyObservers();
    }

    /**
     * Handles the error for the async call to the 
     * master when retrieving the picture.
     * @see org.apache.xmlrpc.AsyncCallback#handleError(java.lang.Exception, java.net.URL, java.lang.String)
     */
    public synchronized void handleError(
        Exception exception,
        URL url,
        String method)
    {
        Log.debug(
            "Error loading image Master url: "
                + url.toString()
                + " Exception: "
                + exception);
        exception.printStackTrace();

    }

}
