/*
 * Created on Oct 27, 2003
 *
 */
package portaview.model;

import java.util.Iterator;
import java.util.List;
import java.util.Observable;
import java.util.Vector;

/**
 * Generic collection class that can be observed.  One needs to use this class
 * if there is a collection of items that they want to observe when added or 
 * removed from this collection.
 * 
 * @author William Lee
 */
public class ObservableCollection extends Observable
{
    private List list = new Vector();

    /**
     * Returns the iterator for this collection.
     * @return iterator for the collection.
     */
    public Iterator iterator()
    {
        return list.iterator();
    }

    /**
     * Clear the collection.
     */
    public void clear()
    {
        list.clear();
        setChanged();
        notifyObservers();
    }
    
    /**
     * Can be called externally to change notify the observers of the observables.
     */
    public void notifyChange()
    {
        setChanged();
        notifyObservers();
    }

    /**
     * Notify the children's observers.  If the instance inside this 
     * collection extends the Observable class, notify the change also.
     */
    public void notifyChildrenObservers()
    {
        for (Iterator i = list.iterator(); i.hasNext();)
        {
            Object obj = i.next();
            if (obj instanceof Observable)
            {
                Observable obs = (Observable)obj;
                obs.notifyObservers();
            }

        }
    }

    /**
     * Add all the element in col to this collecion.
     * @param col collection to add to this collection.
     */
    public void addAll(ObservableCollection col)
    {
        for (Iterator i = col.iterator(); i.hasNext();)
        {
            Object e = (Object) i.next();
            list.add(e);
        }
        setChanged();
        notifyObservers();
    }

    /**
     * Returns the size of this collection.
     * @return int representing the size.
     */
    public int size()
    {
        return list.size();
    }

    /**
     * Removes an object and notify the observing observers.  This will fire the event.
     * @param obj object within this collection to remove.
     */
    public void remove(Object obj)
    {
        list.remove(obj);
        // if we remove this, then we want to notify the observers
        setChanged();
        notifyObservers();
    }

    /**
     * Adds an object to this collection.  This will fire the event.
     * @param obj object to add.
     */
    public void add(Object obj)
    {
        list.add(obj);
        // if we add this, then we want to notify the observers
        setChanged();
        notifyObservers();
    }
}
