/*
 * Created on Sep 20, 2003
 * Copyright (c) 2003 William Lee.  All rights reserved.
 */
package portaview.model;

import java.io.File;
import java.io.FileInputStream;

import portaview.util.ImageUtils;
import portaview.util.StreamUtils;

/**
 * Represents a photo on disk.  Note that this is not the model used in the
 * PortaView manager.  Provides an abstarction to access the pictures on disk.
 * @author <a href="mailto:wwlee1@uiuc.edu">William Lee</a>
 * @version $Id: Photo.java,v 1.3 2003/12/10 06:18:57 wlee Exp $
 */
public class Photo
{
	private String caption;
	private File file;

	public Photo()
	{
		this(null, null);
	}
	
	/**
	 * Constructs a photo with caption.
	 * @param f file that represents the photo
	 * @param caption caption for the photo
	 */
	public Photo(File f, String caption)
	{
		this.file = f;
		this.caption = caption;
	}
	
	/**
	 * Constructs a photo.
	 * @param f file that represents the photo
	 */
	public Photo(File f)
	{
		this.file = f;
		this.caption = f.getName();
	}

	/**
	 * @return the caption of this photo.
	 */
	public String getCaption()
	{
		return caption;
	}

	/**
	 * Sets the caption.
	 * @param caption caption to set.
	 */
	public void setCaption(String caption)
	{
		this.caption = caption;
	}

	/**
	 * @return the file associated with this Photo.
	 */
	public File getFile()
	{
		return file;
	}

	/**
	 * Sets the file associated with this Photo.
	 * @param file file to set.
	 */
	public void setFile(File file)
	{
		this.file = file;
	}

	/**
	 * @return the jpeg encoding of this photo.
	 */
	public byte[] getJpeg()
	{
		if (this.file == null) return null;
		try
		{
			return StreamUtils.getBytesFromStream(new FileInputStream(file));
		}
		catch (Exception e)
		{
			e.printStackTrace();
			return null;
		}
	}
	
	/**
	 * @return the jpeg encoding of this icon.
	 */
	public byte[] getJpegIcon(int width, int height)
	{
		if (this.file == null) return null;
		try
		{
			return ImageUtils.getScaledImage(file, width, height);
		}
		catch (Exception e)
		{
			e.printStackTrace();
			return null;
		}
	}

}
