/*
 * Created on Oct 6, 2003
 * Copyright (c) 2003.  All rights reserved.
 */
package portaview.model;

import java.awt.Image;
import java.util.Observable;
import java.util.Observer;

import portaview.Registry;
import portaview.server.PortaViewClient;

/**
 * Contains the image information within the PortaViewModel.
 * 
 * @author <a href="mailto:wwlee1@uiuc.edu">William Lee</a>
 * @version $Id: PortaViewImageModel.java,v 1.2 2003/12/10 06:18:57 wlee Exp $
 */
public class PortaViewImageModel extends ImageModel implements Observer
{
    private Image image = null;

    /**
     * Used for checking whether the image has changed or not.
     */
    private String currSlide = null;
    private int currIndex = 0;

    /**
     * Constructs the image model within the PortaViewModel.
     * @param dmodel the PortaViewModel that contains this image model.
     */
    public PortaViewImageModel(PortaViewModel dmodel)
    {
        dmodel.addObserver(this);
        addObserver(dmodel);
    }

    /**
     * When the slide setting has changed, then it'll call this method to update the current image.
     * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
     */
    public void update(Observable arg0, Object arg1)
    {
        if (arg1 == null)
            return;
        if (arg1.equals(PortaViewModel.SETTINGS_CHANGED))
        {
            PortaViewModel dm = (PortaViewModel) arg0;
            String arg = (String) arg1;
            String newSlide = dm.getSlideSettings().getSlide();
            int newIndex = dm.getSlideSettings().getIndex();
            if (needToRenewImage(newSlide, newIndex))
            {
                PortaViewClient client = Registry.getClient();
                client.getPhotoFromMaster(dm.getSlideSettings(), this);
            }
        }
    }

    /**
     * Returns true if we need to renew the image.
     * @param newSlide slide name.
     * @param newIndex index of the picture you want to check for renewal.
     * @return true if need renew, false otherwise.
     */
    private boolean needToRenewImage(String newSlide, int newIndex)
    {
        if (newSlide == null)
            return false;
        if (currSlide == null && newSlide != null)
            return true;
        return ((currSlide.equals(newSlide) && (newIndex == currIndex)));
    }
}
