/*
 * Created on Oct 12, 2003
 * Copyright (c) 2003.  All rights reserved.
 */
package portaview.server;

import java.io.File;
import java.util.List;
import java.util.Vector;

import portaview.model.Album;

/**
 * Represents a photo repository.  A repository is a mechanism for the PortaView program to
 * load the picutres from the disk.
 * 
 * @author <a href="mailto:wwlee1@uiuc.edu">William Lee</a>
 * @version $Id: Repository.java,v 1.2 2003/12/10 06:18:57 wlee Exp $
 */
public class Repository
{
	private File albumDir = null;

	/**
	 * Creates a photo repository.
	 * @param albumDir
	 */
	public Repository(File albumDir)
	{
		this.albumDir = albumDir;
	}

	/**
	 * @return the album directory
	 */
	public File getAlbumDir()
	{
		return albumDir;
	}

	/**
	 * Sets the album directory.
	 * @param albumDir the album directory
	 */
	public void setAlbumDir(File albumDir)
	{
		this.albumDir = albumDir;
	}

	/**
	 * Returns a List of Album that are available in the repository.
	 * @return list of album
	 */
	public List listAlbums()
	{
		List l = new Vector();
		File[] albums = this.albumDir.listFiles();
		for (int i = 0; i < albums.length; i++)
		{
			File dir = albums[i];
			if (dir.isDirectory())
			{
				Album a = new Album(dir);
				if (a.getPhoto(0) != null)
				{
					l.add(a);
				}
			}
		}
		return l;
	}

	/**
	 * Returns an album with a particular name.
	 * @param name name of album
	 * @return the album
	 */
	public Album getAlbum(String name)
	{
		return new Album(new File(this.albumDir, name));
	}

}
