/*
 * Created on Nov 14, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package portaview.server;

import portaview.Log;
import portaview.Registry;
import portaview.model.PortaViewModel;
import portaview.model.SlideSettings;

/**
 * Thread that updates the displayer every seconds and see if you need to 
 * advance to the next slide.
 * @author William Lee
 */
public class SlideRunner extends Thread
{
    private boolean shutdown = false;
    private int POLL_TIME = 1000;
    private long nextRefreshTime = 0;

    /**
     * Polls every second and see whether the current PortaView needs to change its picture.
     */
    public void run()
    {
        while (shutdown == false)
        {
            try
            {
                Thread.sleep(POLL_TIME);
                if (shutdown)
                    break;
                PortaViewModel pm =
                    Registry.getMainApplication().getThisPortaViewModel();
                SlideSettings ss = pm.getSlideSettings();
                // if the slide is in pause mode, we just want to go on.
                if (ss.getPlayState() == SlideSettings.PAUSE)
                    continue;
                // Otherwise, test for the time whether the time you 
                // need to refresh 
                if (System.currentTimeMillis() > nextRefreshTime)
                {
                    Log.debug("Time's up at " + System.currentTimeMillis());
                    // advance to the next slide.
                    pm.nextSlide();
                    // set the next notify time
                    nextRefreshTime =
                        System.currentTimeMillis() + (ss.getSpeed() * 1000);
                }
            }
            catch (InterruptedException e)
            {
                if (shutdown)
                    break;
            }
        }
    }
    
    /**
     * Sets the next refresh time.
     */
    public void setNextRefreshTime(long time)
    {
        nextRefreshTime = time;
    }

    /**
     * Shutdowns the thread.
     */
    public void shutdown()
    {
        shutdown = true;
        this.interrupt();
    }

}
