/*
 * Created on Oct 29, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package portaview.view;

import java.awt.Dimension;
import java.awt.Image;
import java.awt.event.MouseEvent;

import javax.swing.TransferHandler;
import javax.swing.event.MouseInputAdapter;

import portaview.Registry;
import portaview.dnd.AlbumTransferHandler;
import portaview.model.AlbumModel;

/**
 * The album icons that represents the album.
 * @author William Lee
 */
public class AlbumIconView extends IconView
{
    private AlbumModel albumModel = null;
    private static final int WIDTH = 120;
    private static final int HEIGHT = 100;

    /**
     * Constrcts an icon view.
     * @param amodel the album model associated with this view.
     * @param apanel the panel that list out the icons of this icon view.
     */
    public AlbumIconView(AlbumModel amodel, AlbumPanel apanel)
    {
        super(apanel);
        this.albumModel = amodel;
        this.albumModel.addObserver(this);
        installDnD();
        setPreferredSize(new Dimension(WIDTH, HEIGHT));
        refresh();
    }

    /**
     * Adds the drag and drop support for the icon view.
     */
    protected void installDnD()
    {
        setTransferHandler(new AlbumTransferHandler());
        DnDStarter starter = new DnDStarter();
        addMouseListener(starter);
        addMouseMotionListener(starter);
    }

    class DnDStarter extends MouseInputAdapter
    {
        public void mousePressed(MouseEvent e)
        {
            TransferHandler th = AlbumIconView.this.getTransferHandler();
            th.exportAsDrag(AlbumIconView.this, e, TransferHandler.COPY);
        }
    }

    /**
     * Returns the icon image.
     * @see portaview.view.IconView#getIconImage()
     */
    public Image getIconImage()
    {
        return albumModel.getImage();
    }

    /**
     * Returns the icon label.
     * @see portaview.view.IconView#getIconLabel()
     */
    public String getIconLabel()
    {
        String[] lbl =
            {
                albumModel.getName(),
                "(" + albumModel.getPhotos().size() + " pictures)" };
        return (getMultiLineLabel(lbl));
    }
    /**
     * @return the album model
     */
    public AlbumModel getAlbumModel()
    {
        return albumModel;
    }

    /**
     * Sets the album model.
     * @param model model to set.
     */
    public void setAlbumModel(AlbumModel model)
    {
        albumModel = model;
    }

    /**
     * Returns true if the displayer is not active
     * @see portaview.view.IconView#needRefresh()
     */
    protected boolean needRefresh()
    {
        return (!Registry.getMainApplication().isDisplayActive());
    }

    /**
     * Sets the tool tips information
     */
    protected void doRefresh()
    {
        String tipText =
            "Album "
                + albumModel.getName()
                + " has "
                + albumModel.getPhotos().size()
                + " photos";
        setToolTipText(tipText);
    }

}
