/*
 * Created on Oct 27, 2003
 */
package portaview.view;

import java.awt.Color;
import java.awt.Component;

import javax.swing.JScrollPane;

import portaview.model.ObservableCollection;

/**
 * Implements the generic scoll pane for icons.
 * @author William Lee
 */
public abstract class IconPanel extends JScrollPane
{

    private IconPanelContent content = null; // contents of the icons.

    public IconPanel()
    {
        this(new ObservableCollection());
    }

    /**
     * Constructs a panel based on an observable list of items IconViews.
     * @param olist
     */
    public IconPanel(ObservableCollection olist)
    {
        super(
            JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
            JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        content = new IconPanelContent(this, olist);
        setBackground(Color.white);
        setViewportView(content);
        getViewport().addChangeListener(content);
    }

    /**
     * Selects the first item in the panel.
     *
     */
    public void selectFirst()
    {
        content.selectFirst();
    }

    /**
     * Refreshes the panel.
     */
    public void refresh()
    {
        content.refresh();
        repaint();
    }

    public void repaint()
    {
        super.repaint();
        if (content != null)
            content.repaint();
    }

    /**
     * Children should implement this in order to get a view from the model.
     * @param model model associated with the view.
     * @param the icon panel (the view needs to refresh the panel as well)
     * @return the view object that can be set in this IconPanel.
     */
    public abstract Object getViewFromModel(Object model, IconPanel ipanel);

    /**
     * Clean up the view when it is discarded.  It should remove all listeners 
     * so they won't leak.
     * @param view the view to remove.
     */
    public abstract void removeView(Object view);
    
    /**
     * Decorates the first view if necessary.  Children can override this
     * method to adds its own decoration.
     */
    public Component decorateFirstView(Component comp)
    {
        return comp;
    }
    
    /**
     * Decorates the second view if necessary.  Children can override this
     * method to adds its own decoration.  By default this returns the component
     * that got passed into this.
     * @param comp component to decorate.
     */
    public Component decorateSecondView(Component comp)
    {
        return comp;
    }
}
