/*
 * Created on Oct 27, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package portaview.view;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.util.Iterator;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import javax.swing.ButtonGroup;
import javax.swing.JPanel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import portaview.model.ObservableCollection;

/**
 * Generic implementation for the content that lives in the IconPanel.  This contains
 * the collection of views and is scrollable.
 * 
 * @author William Lee
 */
public class IconPanelContent
    extends JPanel
    implements ChangeListener, Observer
{
    /**
     * Try to fit the icons based on the width of the viewport.
     */
    public static int FIT_WIDTH = 0;
    /**
     * Try to fit the icons based on the height of the viewport.
     */
    public static int FIT_HEIGHT = 1;

    private int mode = FIT_WIDTH;
    private ObservableCollection models = null;
    private List views = null;
    private IconPanel panel = null;

    /**
     * The button group used to control the select behavior.
     */
    private ButtonGroup buttonGroup = new ButtonGroup();

    /**
     * Selects the first view.
     *
     */
    public void selectFirst()
    {
        select(0);
    }

    /**
     * Selects the nth element.
     * @param n the index to select.
     */
    public void select(int n)
    {
        int sel = n;
        if (views == null)
            return;
        if (views.size() == 0)
            return;
        if (sel >= views.size())
            sel = 0;
        IconView iv = (IconView) views.get(sel);
        if (iv == null)
            return;
        iv.doClick();
    }

    public IconPanelContent(IconPanel panel, ObservableCollection models)
    {
        this.panel = panel;
        this.models = models;
        this.views = new Vector();
        FlowLayout layout = new FlowLayout();
        layout.setAlignment(FlowLayout.LEFT);
        setLayout(layout);
        models.addObserver(this);
        refresh();
    }

    /**
     * Refreshes the content of this view.
     */
    public void refresh()
    {
        int lastSelected = 0;
        int ctr = 0;
        IconView sampleIcon = null;
        for (Iterator iter = views.iterator(); iter.hasNext();)
        {
            IconView iv = (IconView) iter.next();
            if (iv.isSelected())
                lastSelected = ctr;
            panel.removeView(iv);
            buttonGroup.remove(iv);
            ctr++;
        }
        // remove all the content
        removeAll();
        boolean isFirst = true;
        boolean isSecond = false;
        for (Iterator iter = models.iterator(); iter.hasNext();)
        {
            IconView icon =
                (IconView) panel.getViewFromModel(iter.next(), panel);
            views.add(icon);
            if (isFirst)
            {
                add(panel.decorateFirstView(icon));
                isFirst = false;
                isSecond = true;
            }
            else if (isSecond)
            {
                add(panel.decorateSecondView(icon));
                isSecond = false;
            }
            else
            {
                add(icon);
            }
            buttonGroup.add(icon);
            icon.refresh();
            sampleIcon = icon;
        }
        select(lastSelected);
        /*
        setPreferredSize(
            new Dimension(panel.getViewport().getWidth(), getDesiredHeight()));
            */
        int width = 100;
        int height = 80 * models.size();
        if (sampleIcon != null)
        {
            width = sampleIcon.getWidth();
            height = (sampleIcon.getHeight() * models.size()) + 20;
        }
        setPreferredSize(new Dimension(width, height));
    }

    /**
     * Recalculate the row of icons in order to fit the view port better.
     * 
     * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
     */
    public void stateChanged(ChangeEvent e)
    {
        //refresh();
    }

    /**
     * Listen to the event if the observable has changed.
     * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
     */
    public void update(Observable o, Object arg)
    {
        refresh();
    }

}
