/*
 * Created on Oct 29, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package portaview.view;

import java.awt.Image;
import java.util.Observable;
import java.util.Observer;

import javax.swing.AbstractButton;
import javax.swing.ImageIcon;
import javax.swing.JToggleButton;
import javax.swing.SwingConstants;

import portaview.Log;

/**
 * Icon view that serve as the parent of things that can have an icon.
 * @author William Lee
 */
public abstract class IconView extends JToggleButton implements Observer
{
    public static final int DEFAULT_WIDTH = 100;
    public static final int DEFAULT_HEIGHT = 100;

    private ImageIcon icon = null;
    private IconPanel iconPanel = null;

    public IconView(IconPanel ipanel)
    {
        super();
        iconPanel = ipanel;
        setHorizontalAlignment(SwingConstants.CENTER);

    }
    /**
     * Update to call when the PortaView has changed.
     * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
     */
    public void update(Observable o, Object arg)
    {
        refresh();
    }

    /**
     * Children should implement this in order to get the icon image.
     * @return
     */
    protected abstract Image getIconImage();

    /**
     * Children should implement this in order to return the text label.
     * @return
     */
    protected abstract String getIconLabel();

    /**
     * Children can override this method in order to do its specific refresh.
     *
     */
    protected void doRefresh()
    {
    }

    /**
     * Returns true if the view need refreshing.
     *
     */
    protected abstract boolean needRefresh();
    
    /**
     * Children class can override this class to process the icon.
     * @param img the image to process (can apply the effects for the icon).
     */
    protected Image postProcessIcon(Image img)
    {
        return img;
    }

    /**
     * Refreshes the view.
     */
    public void refresh()
    {
        if (!needRefresh())
            return;
        // should get the icon image from the cache, but oh well.
        try
        {
            Image img = getIconImage();
            if (img != null)
            {
                icon = new ImageIcon(postProcessIcon(img));
                setIcon(icon);
            }
        }
        catch (Exception e)
        {
            Log.debug(e.toString());
        }
        setText(getIconLabel());
        setVerticalTextPosition(AbstractButton.BOTTOM);
        setHorizontalTextPosition(AbstractButton.CENTER);
        doRefresh();
        repaint();
        iconPanel.repaint();
    }

    /**
     * Utility method to get a multi-line label string.
     * @param lines lines of text
     * @return an HTML encoded label text that you can use in a multi-line label.
     */
    protected String getMultiLineLabel(String[] lines)
    {
        StringBuffer rtn = new StringBuffer();
        rtn.append("<html>");
        //   rtn.append("<center>");
        for (int i = 0; i < lines.length; i++)
        {
            rtn.append(lines[i]);
            if (i != lines.length - 1)
                rtn.append("<br>");
        }
        // rtn.append("</center>");
        rtn.append("</html>");
        return rtn.toString();
    }

}
