/*
 * Created on Sep 30, 2003
 * Copyright (c) 2003.  All rights reserved.
 */
package portaview.view;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;

import portaview.AbstractViewer;
import portaview.Displayer;
import portaview.Registry;
import portaview.model.SlideSettings;

/**
 * Represents the button bar that controls the images.
 * 
 * @author <a href="mailto:wwlee1@uiuc.edu">William Lee</a>
 * @version $Id: ImageControlBar.java,v 1.13 2003/12/10 06:18:57 wlee Exp $
 */
public class ImageControlBar extends JPanel
{
    private AbstractViewer viewer = null;
    private static int CL_WIDTH = 100;
    private static int CL_HEIGHT = 40;
    private JButton prevButton = null;
    private JButton pauseButton = null;
    private JButton playButton = null;
    private JButton nextButton = null;
    private JButton settingsButton = null;

    /**
     * Mode for the preview.
     */
    public static int PREVIEW = 0;

    /**
     * Mode for display.
     */
    public static int DISPLAY = 1;

    public ImageControlBar(int mode, AbstractViewer av)
    {
        super();
        setLayout(new BorderLayout());
        JPanel buttonPanel = new JPanel();
        this.viewer = av;
        // Create a button that enters or leaves full-screen mode
        JButton btn = null;
        if (mode == DISPLAY)
        {
            btn = new JButton("Switch to Manager View");
            btn.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    Displayer disp = (Displayer) (viewer);
                    disp.setVisible(false);
                    Registry.getMainApplication().setDisplayActive(false);
                }
            });
            buttonPanel.add(btn);
        }

        prevButton = new JButton(createImageIcon("backward.gif"));
        prevButton.setToolTipText("Previous Picture");
        prevButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                viewer.prevSlide();
            }
        });
        buttonPanel.add(prevButton);
        pauseButton = new JButton(createImageIcon("pause.gif"));
        pauseButton.setToolTipText("Pause");
        pauseButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                viewer.pause();
            }
        });
        buttonPanel.add(pauseButton);
        playButton = new JButton(createImageIcon("play.gif"));
        playButton.setToolTipText("Play");
        playButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                viewer.play();
            }
        });
        buttonPanel.add(playButton);
        nextButton = new JButton(createImageIcon("forward.gif"));
        nextButton.setToolTipText("Next Picture");
        nextButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                viewer.nextSlide();
            }
        });
        buttonPanel.add(nextButton);
        if (mode != DISPLAY)
        {
            settingsButton = new JButton("Border and Play Speed Settings...");
            buttonPanel.add(settingsButton);
            settingsButton.addActionListener(new ActionListener()
            {

                public void actionPerformed(ActionEvent e)
                {
                    SettingsDialog sd =
                        new SettingsDialog(
                            Registry.getMainApplication(),
                            Registry
                                .getMainApplication()
                                .getPreview()
                                .getModel());
                    sd.pack();
                    sd.show();
                }
            });
        }
        add(buttonPanel, BorderLayout.CENTER);
    }

    /**
     * Creates the image icon based on the path.
     * @param imgName the name of the image.
     * @return the image icon.
     */
    protected ImageIcon createImageIcon(String imgName)
    {
        java.net.URL imgURL =
            ImageControlBar.class.getResource("/portaview/images/" + imgName);
        if (imgURL == null)
            return null;
        return new ImageIcon(imgURL);
    }

    /**
     * Returns a new instance of ImageControlBar for the preview panel.
     * @param av The controller that this bar can link its buttons to.
     * @return the ImageControlBar for the preview panel.
     */
    public static ImageControlBar getPreviewBar(AbstractViewer av)
    {
        return new ImageControlBar(PREVIEW, av);
    }

    /**
     * Returns a new instance of ImageControlBar for the display panel
     * @param av The controller that this bar can link its buttons to.
     * @return the ImageControlBar for the display (full-screen) panel.
     */
    public static ImageControlBar getDisplayBar(AbstractViewer av)
    {
        return new ImageControlBar(DISPLAY, av);
    }

    /**
     * Sets the label for the slide settings.
     * @param ss slide settings
     */
    public void setImageCount(SlideSettings ss)
    {

    }
    /**
     * @return the next button.
     */
    public JButton getNextButton()
    {
        return nextButton;
    }

    /**
     * @return the pause button
     */
    public JButton getPauseButton()
    {
        return pauseButton;
    }

    /**
     * @return the play button
     */
    public JButton getPlayButton()
    {
        return playButton;
    }

    /**
     * @return the previous button
     */
    public JButton getPrevButton()
    {
        return prevButton;
    }

    /**
     * Sets next button.
     * @param button to set.
     */
    public void setNextButton(JButton button)
    {
        nextButton = button;
    }

    /**
     * Sets pause button.
     * @param button to set.
     */
    public void setPauseButton(JButton button)
    {
        pauseButton = button;
    }

    /**
     * Sets play button.
     * @param button to set.
     */
    public void setPlayButton(JButton button)
    {
        playButton = button;
    }

    /**
     * Sets play button.
     * @param button to set.
     */
    public void setPrevButton(JButton button)
    {
        prevButton = button;
    }

    /**
     * @return the settings button
     */
    public JButton getSettingsButton()
    {
        return settingsButton;
    }

    /**
     * Sets the settings button
     * @param button button to set.
     */
    public void setSettingsButton(JButton button)
    {
        settingsButton = button;
    }

}
