/*
 * Created on Oct 27, 2003
 *
 */
package portaview.view;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import portaview.Displayer;
import portaview.Registry;
import portaview.model.PortaViewCollection;
import portaview.model.PortaViewModel;

/**
 * Represents a PortaViewPanel that list out the PortaViews on the left.
 * @author William Lee
 */
public class PortaViewPanel extends IconPanel
{
    private PortaViewCollection portaViews = null;
    private List portaViewViews = null;

    /**
     * Given a list of PortaView models, draw the view.
     * @param portaViews
     */
    public PortaViewPanel(PortaViewCollection portaViews)
    {
        super(portaViews);
    }

    /**
     * Adds an component before the first view in the panel.
     */
    public Component decorateFirstView(Component comp)
    {
        JPanel rtn = new JPanel(new BorderLayout());
        rtn.add(comp, BorderLayout.CENTER);
        JLabel lbl = new JLabel("This PortaView:");
        lbl.setHorizontalAlignment(SwingConstants.CENTER);
        lbl.setVerticalAlignment(SwingConstants.CENTER);
        rtn.add(lbl, BorderLayout.NORTH);

        JButton fullScreen = new JButton("Full Screen View");
        fullScreen.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                Displayer disp = Registry.getMainApplication().getDisplayer();
                disp.setVisible(true);
                disp.getView().refreshImage();
                Registry.getMainApplication().setDisplayActive(true);
            }
        });
        rtn.add(fullScreen, BorderLayout.SOUTH);
        return rtn;
    }
    
    /**
     * Decorates the second view.
     * @param comp the PortaViewView to decorate
     * @return the component with a label that says "Other PortaView(s):"
     */
    public Component decorateSecondView(Component comp)
    {
        JPanel rtn = new JPanel(new BorderLayout());
        rtn.add(comp, BorderLayout.CENTER);
        JLabel lbl = new JLabel("Other PortaView(s):");
        lbl.setHorizontalAlignment(SwingConstants.CENTER);
        lbl.setVerticalAlignment(SwingConstants.CENTER);
        rtn.add(lbl, BorderLayout.NORTH);
        rtn.setBorder(BorderFactory.createEmptyBorder(20,0,0,0));
        return rtn;
    }

    /**
     * Generates a PortaViewView from the PortaViewModel.
     * @see portaview.view.IconPanel#getViewFromModel(java.lang.Object)
     */
    public Object getViewFromModel(Object model, IconPanel ipanel)
    {
        PortaViewModel element = (PortaViewModel) model;
        PortaViewView view =
            new PortaViewView(element, (PortaViewPanel) ipanel);
        return view;
    }

    /**
     * Remove the view and clean up the observers.
     * @see portaview.view.IconPanel#removeView(java.lang.Object)
     */
    public void removeView(Object view)
    {
        PortaViewView pvv = (PortaViewView) view;
        pvv.getPortaViewModel().deleteObserver(pvv);
    }
}
