/*
 * Created on Nov 13, 2003
 */
package portaview.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Dictionary;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import portaview.Registry;
import portaview.model.PortaViewModel;
import portaview.model.SlideSettings;

/**
 * Represents a dialog that one can set the slide settings.
 * @author William Lee
 */
public class SettingsDialog extends JDialog
{
    private PortaViewModel pvm = null;
    private JSlider speedSlider = null;
    private JSlider borderSlider = null;
    private JSlider transSlider = null;
    private JPanel transPanel = null;
    private JButton colorButton = null;
    private JPanel colorButtonPanel = null;
    private SettingsPreview previewPanel = null;

    public SettingsDialog(JFrame parent, PortaViewModel m)
    {
        super(parent, "Border and Play Settings for " + m.getName(), true);
        pvm = m;
        // calculate where to put the dialog.  The guideline says one
        // should place the dialog at approx. 1/5 below main application.
        Point p = new Point(Registry.getMainApplication().getLocation());
        Dimension d = new Dimension(Registry.getMainApplication().getSize());
        p.x = p.x + (d.width / 3) + 50;
        p.y = p.y + (d.height / 5);
        setLocation(p);
        setResizable(false);
        getContentPane().setLayout(new BorderLayout());
        initSliders();
        initButtons();
    }

    private void initButtons()
    {
        final JDialog diag = this;
        final JSlider sslider = speedSlider;
        final JSlider bslider = borderSlider;
        JButton okButton = new JButton("OK");
        okButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                pvm.getSlideSettings().setSpeed(sslider.getValue());
                pvm.getSlideSettings().setBorder(bslider.getValue());
                pvm.getSlideSettings().setBorderColor(
                    colorButton.getBackground());
                pvm.getSlideSettings().setTransparency(transSlider.getValue());
                pvm.forceNotifyObservers();
                diag.dispose();
            }
        });
        JButton cButton = new JButton("Cancel");
        cButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                diag.dispose();
            }
        });
        JPanel p = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        p.add(okButton);
        p.add(cButton);
        getContentPane().add(p, BorderLayout.SOUTH);
    }

    /**
     * Initialize the sliders.
     *
     */
    private void initSliders()
    {
        SlideSettings ss = pvm.getSlideSettings();
        JPanel speedPanel = initSpeedPanel(ss);
        JPanel borderPanel = initMainBorderPanel(pvm);
        JPanel content = new JPanel(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;
        c.fill = GridBagConstraints.BOTH;
        content.add(speedPanel, c);
        c.gridx = 0;
        c.gridy = 1;
        c.fill = GridBagConstraints.BOTH;
        content.add(borderPanel, c);
        getContentPane().add(content, BorderLayout.CENTER);
    }

    private JPanel initMainBorderPanel(PortaViewModel pvm)
    {
        SlideSettings ss = pvm.getSlideSettings();
        JPanel borderPanel = new JPanel(new GridBagLayout());
        borderPanel.setBorder(new TitledBorder("Picture Border"));
        // note that we need to initialize the transPanel and getColorPanel first
        // for we need to use the sliders in the borderSliderPanel
        JPanel transPanel = initTransPanel(ss);
        JPanel getColorPanel = initColorPanel(ss);
        JPanel borderSliderPanel = initBorderPanel(ss);
        JPanel ppanel = initPreviewPanel(pvm);

        GridBagConstraints c = new GridBagConstraints();
        // put the border panel
        c.gridx = 0;
        c.gridy = 0;
        c.fill = GridBagConstraints.BOTH;
        borderPanel.add(borderSliderPanel, c);
        // put the preview panel
        c.gridx = 1;
        c.gridy = 0;
        c.gridheight = 3;
        c.gridwidth = 1;
        c.fill = GridBagConstraints.NONE;
        c.anchor = GridBagConstraints.CENTER;
        c.insets = new Insets(10, 10, 10, 10);
        borderPanel.add(ppanel, c);
        // put the other two
        c.gridx = 0;
        c.gridy = 1;
        c.gridheight = 1;
        c.gridwidth = 1;
        c.fill = GridBagConstraints.BOTH;
        c.insets = new Insets(0, 0, 0, 0);
        borderPanel.add(getColorPanel, c);
        c.gridx = 0;
        c.gridy = 2;
        c.gridheight = 1;
        c.gridwidth = 1;
        c.fill = GridBagConstraints.BOTH;
        c.ipadx = 0;
        c.ipady = 0;
        borderPanel.add(transPanel, c);
        return borderPanel;
    }

    private JPanel initPreviewPanel(PortaViewModel pvm)
    {
        previewPanel =
            new SettingsPreview(
                pvm.getImageModel().getImage(),
                borderSlider,
                transSlider,
                colorButton);
        borderSlider.addChangeListener(previewPanel);
        transSlider.addChangeListener(previewPanel);
        return previewPanel;
    }

    private JPanel initTransPanel(SlideSettings ss)
    {
        // panel for getting transparency.
        transPanel = new JPanel();
        transPanel.setBorder(new TitledBorder("Border Transparency"));
        transSlider = new JSlider(1, 10, ss.getTransparency());
        transSlider.putClientProperty("JSlider.isFilled", Boolean.TRUE);
        transSlider.setPaintTicks(true);
        transSlider.setMinorTickSpacing(1);
        transSlider.setPaintLabels(true);
        transSlider.setSnapToTicks(true);

        // make the custom label
        Dictionary tlabels = new Hashtable();
        tlabels.put(new Integer(1), new JLabel("Transparent"));
        tlabels.put(new Integer(10), new JLabel("Solid"));
        transSlider.setLabelTable(tlabels);
        transPanel.add(transSlider);
        return transPanel;
    }

    private JPanel initColorPanel(SlideSettings ss)
    {
        colorButtonPanel = new JPanel(new GridBagLayout());
        colorButtonPanel.setBorder(new TitledBorder("Border Color"));
        GridBagConstraints cc = new GridBagConstraints();
        cc.fill = GridBagConstraints.BOTH;
        cc.gridx = 0;
        cc.gridy = 0;
        cc.weightx = 1;
        cc.weighty = 1;
        colorButton = new JButton(" ");
        colorButton.setBackground(ss.getBorderColor());
        colorButton.setFocusable(false);
        colorButtonPanel.add(colorButton, cc);
        final JButton getColor = colorButton;
        getColor.addActionListener(new ActionListener()
        {

            public void actionPerformed(ActionEvent e)
            {
                Color c;
                c =
                    JColorChooser.showDialog(
                        ((Component) e.getSource()).getParent(),
                        "Border Color",
                        getColor.getBackground());
                getColor.setBackground(c);
                // repaint the preview panel.
                previewPanel.repaint();

            }
        });
        return colorButtonPanel;
    }

    private JPanel initBorderPanel(SlideSettings ss)
    {
        JPanel borderSliderPanel = new JPanel();
        borderSliderPanel.setBorder(new TitledBorder("Border Width"));
        borderSlider = new JSlider(0, 16, ss.getBorder());
        borderSlider.putClientProperty("JSlider.isFilled", Boolean.TRUE);
        borderSlider.setPaintTicks(true);
        borderSlider.setMinorTickSpacing(2);
        borderSlider.setPaintLabels(true);
        borderSlider.setSnapToTicks(true);

        // make the custom label
        Dictionary labels = new Hashtable();
        labels.put(new Integer(0), new JLabel("None"));
        labels.put(new Integer(8), new JLabel("Medium"));
        labels.put(new Integer(16), new JLabel("Thick"));
        borderSlider.setLabelTable(labels);
        borderSliderPanel.add(borderSlider);
        // add the listener so we can disable the color button and the 
        // transparency slider if border is set to none
        borderSlider.addChangeListener(new ChangeListener()
        {
            public void stateChanged(ChangeEvent e)
            {
                updateBorderSldiers();
            }
        });
        updateBorderSldiers();
        return borderSliderPanel;
    }

    /**
     * Disables or enables the color button and transparency slider 
     * according to the value of the border slider.
     *
     */
    private void updateBorderSldiers()
    {
        if (borderSlider.getValue() == 0)
        {
            // disable the buttons and the sliders for transparency
            colorButton.setEnabled(false);
            colorButton.setBackground(Color.GRAY);
            colorButtonPanel.setEnabled(false);
            ((TitledBorder) colorButtonPanel.getBorder()).setTitleColor(
                Color.GRAY);
            transSlider.setEnabled(false);
            transPanel.setEnabled(false);
            ((TitledBorder) transPanel.getBorder()).setTitleColor(Color.GRAY);
            disableLabels(transSlider);
        }
        else
        {
            // enable the buttons and the sliders for transparency
            colorButton.setEnabled(true);
            colorButtonPanel.setEnabled(true);
            ((TitledBorder) colorButtonPanel.getBorder()).setTitleColor(
                Color.BLACK);
            transSlider.setEnabled(true);
            transPanel.setEnabled(true);
            ((TitledBorder) transPanel.getBorder()).setTitleColor(Color.BLACK);
            enableLabels(transSlider);
        }
    }

    /**
     * Disables the labels here
     * @param slider
     */
    private void disableLabels(JSlider slider)
    {
        changeLabelsColor(slider, Color.GRAY);
    }

    /**
     * Enables the labels for the slider.
     * @param slider
     */
    private void enableLabels(JSlider slider)
    {
        changeLabelsColor(slider, Color.BLACK);
    }

    private void changeLabelsColor(JSlider slider, Color c)
    {
        Dictionary labels = slider.getLabelTable();
        for (Enumeration en = labels.keys(); en.hasMoreElements();)
        {
            JLabel l = (JLabel) labels.get(en.nextElement());
            l.setForeground(c);
        }
    }

    private JPanel initSpeedPanel(SlideSettings ss)
    {
        JPanel speedPanel = new JPanel(new BorderLayout());
        speedPanel.setBorder(new TitledBorder("Album Play Speed (seconds)"));
        speedSlider = new JSlider(10, 30, ss.getSpeed());
        speedSlider.putClientProperty("JSlider.isFilled", Boolean.TRUE);
        speedSlider.setPaintTicks(true);
        speedSlider.setMinorTickSpacing(5);

        Dictionary slabels = new Hashtable();
        slabels.put(new Integer(5), new JLabel("5"));
        slabels.put(new Integer(10), new JLabel("10"));
        slabels.put(new Integer(15), new JLabel("15"));
        slabels.put(new Integer(20), new JLabel("20"));
        slabels.put(new Integer(25), new JLabel("25"));
        slabels.put(new Integer(30), new JLabel("30"));
        speedSlider.setLabelTable(slabels);

        speedSlider.setPaintLabels(true);
        speedSlider.setSnapToTicks(true);
        speedPanel.add(speedSlider);
        return speedPanel;
    }

}
