/*
 * Created on Nov 20, 2003
 */
package portaview.view;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.image.BufferedImage;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import portaview.util.ImageUtils;

/**
 * Simple panel that shows preview for the settings used in the SettingsDialog.
 * @author William Lee
 */
public class SettingsPreview extends JPanel implements ChangeListener 
{
    private BufferedImage image = null;
    private JSlider borderSlider = null;
    private JSlider transSlider = null;
    private JButton colorButton = null;
    
    private BufferedImage origImage = null;
    private static final int PREVIEW_WIDTH = 160;
    private static final int PREVIEW_HEIGHT = 128;

    public SettingsPreview(
        Image img,
        JSlider bs,
        JSlider ts,
        JButton cbut)
    {
        this.borderSlider = bs;
        this.transSlider = ts;
        this.colorButton = cbut;
        try
        {
            image =
                ImageUtils.getBufferedImageInFrame(
                    ImageUtils.getBufferedImage(img, this),
                    PREVIEW_WIDTH,
                    PREVIEW_HEIGHT,
                    this);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        setPreferredSize(new Dimension(image.getWidth(), image.getHeight()));
    }

    /**
     * Draws the preview graphics with all the effects.
     */
    public void paintComponent(Graphics g)
    {
        super.paintComponent(g);
        Graphics2D g2 = (Graphics2D) g;
        image.createGraphics();
        g2.drawImage(image, 0, 0, this);
        // tries to draw the border
        ImageUtils.drawBorderRatio(
            g2,
            (transSlider.getValue() / 10.0F),
            borderSlider.getValue(),
            image.getWidth(),
            image.getHeight(),
            colorButton.getBackground());
    }

    /**
     * Should update myself when things have changed.
     * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
     */
    public void stateChanged(ChangeEvent e)
    {
        repaint();
    }

}
