/*
 * Created on Sep 28, 2003
 * Copyright (c) 2003.  All rights reserved.
 */
package test.portaview;

/**
 * 
 * @author <a href="mailto:wwlee1@uiuc.edu">William Lee</a>
 * @version $Id: TestDisplayer.java,v 1.8 2003/10/29 02:47:19 wlee Exp $
 */

import java.awt.Color;
import java.awt.Image;
import java.io.File;
import java.io.FileInputStream;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.apache.xmlrpc.XmlRpcClient;

import portaview.Displayer;
import portaview.Registry;
import portaview.model.Album;
import portaview.model.SlideSettings;
import portaview.server.PortaViewServer;
import portaview.util.ImageUtils;
import portaview.util.StreamUtils;

public class TestDisplayer extends TBase
{

    public TestDisplayer(String name)
    {
        super(name);
    }

    /**
     * Deallocates the resources
     */
    protected void tearDown()
    {
        super.tearDown();
    }

    protected void setUp()
    {
        super.setUp();
    }

    /** 
     * Constructs a test suite.
     */
    public static Test suite()
    {
        return new TestSuite(TestDisplayer.class);
    }

    public void testMain()
    {
        tRegisterRemotePortaView();
        tViewerServerGetImage();
        tViewerGetAlbums();
        tUpdatePortaView();
    }

    /**
     * Gets the image from XMLRPC and compare with the one loaded from disk.
     */
    private void tViewerServerGetImage()
    {
        try
        {
            Vector params = new Vector();
            params.addElement("slides");
            params.addElement(new Integer(0));
            byte[] result = (byte[]) xmlExec("main.getPhoto", params);
            assertTrue(result != null);
            assertFileEqualsBytes(getRealImage("/slides/0.jpg"), result);
        }
        catch (Exception e)
        {
            generalFail(e);
        }
    }

    /**
     * Returns the real image path.
     * @param relPath relative path to the image.
     * @return the actual File for the image.
     */
    private File getRealImage(String relPath)
    {
        return new File(TEST_IMG_BASE + relPath);
    }

    /**
     * Returns true if the image file
     * @param f
     * @return
     */
    private void assertFileEqualsBytes(File f, byte[] bytes)
    {
        try
        {
            byte[] realThing =
                StreamUtils.getBytesFromStream(new FileInputStream(f));
            assertTrue(
                "results are not equal",
                TUtils.bytesEqual(realThing, bytes));
        }
        catch (Exception e)
        {
            generalFail(e);
        }
    }

    /**
     * Asser that the image file, after resizing to width and height, equals 
     * to the bytes you retrieve.
     * @param f image file
     * @param width width to resize to
     * @param height height to resize to
     * @param bytes bytes to compare to
     */
    private void assertJpegResizeEquals(
        File f,
        int width,
        int height,
        byte[] bytes)
    {
        try
        {
            byte[] compare = ImageUtils.getScaledImage(f, width, height);
            assertNotNull(compare);
            assertTrue(
                "resize image invalid",
                TUtils.bytesEqual(compare, bytes));
        }
        catch (Exception e)
        {
            generalFail(e);
        }
    }

    /**
     * Test to get the albums
     *
     */
    private void tViewerGetAlbums()
    {
        try
        {
            Vector p = new Vector();
            Vector rtn = (Vector) xmlExec("main.getAlbums", p);
            assertNotNull(rtn);
            // The vector should be vectors of name=> String and img => bytes[]
            // verify that there are two albums.
            assertTrue(rtn.size() == 2);
            for (Iterator iter = rtn.iterator(); iter.hasNext();)
            {
                Hashtable slideStruct = (Hashtable) iter.next();
                String name = (String) slideStruct.get(Album.XT_NAME);
                verifyAlbum(name, slideStruct);
            }
        }
        catch (Exception e)
        {
            generalFail(e);
        }
    }

    /**
     * Checks if the album returned is correct.
     * @param album
     * @param ht
     */
    private void verifyAlbum(String album, Hashtable ht)
    {
        // Gets the thumbnail from the album.
        assertTrue(album.equals((String) ht.get(Album.XT_NAME)));
        assertJpegResizeEquals(
            getRealImage("/" + album + "/0.jpg"),
            Album.ICON_W,
            Album.ICON_H,
            (byte[]) ht.get(Album.XT_ICON));
    }

    private void tViewerGetPortaViews()
    {

    }

    /**
     * Executes the XMLRPC call.
     * @param func function to call
     * @param p parameters
     * @return what XMLRPC returns.
     * @throws Exception when there's an XMLRPC error.
     */
    private Object xmlExec(String func, Vector p) throws Exception
    {
        return getClient().execute(func, p);
    }

    /**
     * Returns an instance of the XMLRPC client.
     * @throws Exception
     */
    private XmlRpcClient getClient() throws Exception
    {

        XmlRpcClient xmlrpc =
            new XmlRpcClient("http://localhost:" + PORT + "/RPC2");
        return xmlrpc;
    }

    /**
     * Tests the registration of the panel.
     */
    private void tRegisterRemotePortaView()
    {
        try
        {
            Vector p = new Vector();
            p.add(PNAME);
            p.add(Registry.getServer().getInetAddress().getHostAddress());
            Hashtable rtn = (Hashtable) xmlExec("main.registerMaster", p);
            assertNotNull(rtn);
            // register to itself, should have the same name
            assertTrue("name", rtn.get("name").equals(PNAME));
            assertTrue(
                "ip",
                rtn.get("ip").equals(
                    Registry.getServer().getInetAddress().getHostAddress()));
        }
        catch (Exception e)
        {
            fail(e.toString());
        }
    }

    /**
     * Retrieves the icons for all the albums from the portaview.
     */
    private void tGetAlbumsFromPortaView()
    {
    }

    /**
     * Tests the update function for a PortaView.  The master would call this 
     * on the peer clients.
     */
    private void tUpdatePortaView()
    {
        try
        {
            Vector p = new Vector();
            SlideSettings ss = new SlideSettings();
            ss.setSlide("slides");
            ss.setIndex(0);
            ss.setTotal(3);
            ss.setSpeed(3);
            ss.setBorderColor(new Color(25, 30, 35));
            ss.setBorder(3);
            p.add(ss.toHashtable());
            SlideSettings oldSet =
                Registry
                    .getMainApplication()
                    .getThisPortaViewModel()
                    .getSlideSettings();
            /*
            if (oldSet != null)
            	System.out.println(oldSet.toString());
            	*/
            Boolean rtn = (Boolean) getClient().execute("main.update", p);
            //			rtn = (Boolean)getClient().execute("main.update", p);
            assertTrue(rtn.booleanValue());

            // Check if the image is deployed succesfully.  The update call should 
            // invoke an RPC call to the master to get the image.
            tDisplayer();

            Vector sp = new Vector();
            Hashtable rtnSetHash =
                (Hashtable) getClient().execute("main.getStatus", sp);
            SlideSettings s = new SlideSettings(rtnSetHash);
            //System.out.println(s.toString());
            assertTrue(s.getSlide().equals("slides"));
            // we looped back in tDisplayer, so it's 2
            assertTrue(s.getIndex() == 2);
            assertTrue(s.getTotal() == 3);
            assertTrue(s.getSpeed() == 3);
            assertTrue(s.getBorderColor().getRed() == 25);
            assertTrue(s.getBorderColor().getGreen() == 30);
            assertTrue(s.getBorderColor().getBlue() == 35);
            assertTrue(s.getBorder() == 3);

        }
        catch (Exception e)
        {
            fail(e.toString());
        }
    }

    /**
     * Waits until there is an image.  Timeout in 10 seconds.
     * @return
     * @throws Exception
     */
    private Image waitForImage() throws Exception
    {
        Image img = null;
        int timeout = 0;
        while (true)
        {
            img =
                Registry
                    .getMainApplication()
                    .getThisPortaViewModel()
                    .getImageModel()
                    .getImage();
            if (img != null)
            {
                break;
            }
            if (timeout == 10)
            {
                fail("Timeout while waiting for displayer");
                return null;
            }
            Thread.sleep(1000);
            timeout++;
        }
        return img;
    }

    /**
     * Testing the displayer.  This will load several images and go through them one by one.
     */
    private void tDisplayer()
    {
        try
        {
            // wait until the picture is ready for display
            // check the displayer.  Timeout after 10 seconds
            Image img = waitForImage();
            assertNotNull(img);
            PortaViewServer vs = Registry.getServer();
            Displayer dcon =
                Registry.getMainApplication().getDisplayer();
            dcon.setVisible(true);
            Thread.sleep(2000);
            dcon.nextSlide();
            assertTrue(
                dcon.getModel().getSlideSettings().getIndex() == 1);
            Thread.sleep(2000);
            dcon.nextSlide();
            assertTrue(
                dcon.getModel().getSlideSettings().getIndex() == 2);
            Thread.sleep(2000);
            dcon.nextSlide();
            // loop back!
            assertTrue(
                dcon.getModel().getSlideSettings().getIndex() == 0);
            Thread.sleep(2000);
            dcon.prevSlide();
            // loop back!
            assertTrue(
                dcon.getModel().getSlideSettings().getIndex() == 2);
            Thread.sleep(2000);
            dcon.setVisible(true);
        }
        catch (Exception e)
        {
            e.printStackTrace();
            fail(e.toString());
        }
    }

    /**
     * Creates the main method for the test class to run. 
     */
    public static void main(String[] args)
    {
        junit.textui.TestRunner.run(suite());
    }
}
